package com.jiuqi.sql;

package .tmp;

import com.jiuqi.sql.DBException;
import com.jiuqi.sql.FieldInfo;
import com.jiuqi.sql.FieldInfoList;
import com.jiuqi.sql.OutputRecordBuffer;
import java.util.Date;

public abstract class OutputDataSet {
  private boolean active;
  
  private FieldInfo findField(String fieldName) throws DBException {
    int index = this.fieldInfos.find(fieldName);
    if (index < 0)
      throw new DBException(String.valueOf(fieldName) + " Not Exists"); 
    FieldInfo result = this.fieldInfos.get(index);
    return result;
  }
  
  public FieldInfo addField(String name, int type) {
    FieldInfo fieldInfo = this.fieldInfos.add();
    fieldInfo.name = name;
    fieldInfo.type = type;
    switch (type) {
      case 1:
        fieldInfo.size = 1;
        break;
      case 3:
        fieldInfo.size = 8;
        break;
      case 5:
        fieldInfo.size = 4;
        break;
      case 8:
        fieldInfo.size = 8;
        break;
      case 2:
        fieldInfo.size = 8;
        break;
    } 
    return fieldInfo;
  }
  
  public void addField(String name, int type, int size) {
    FieldInfo fieldInfo = this.fieldInfos.add();
    fieldInfo.name = name;
    fieldInfo.type = type;
    fieldInfo.size = size;
  }
  
  public FieldInfoList getFields() {
    return this.fieldInfos;
  }
  
  private FieldInfoList fieldInfos = new FieldInfoList();
  
  private OutputRecordBuffer buffer;
  
  public abstract byte[] getBytes();
  
  protected abstract void postRecord(OutputRecordBuffer paramOutputRecordBuffer) throws DBException;
  
  protected abstract void internalCreate(FieldInfoList paramFieldInfoList) throws DBException;
  
  protected abstract OutputRecordBuffer internalOpen() throws DBException;
  
  protected abstract void internalClose() throws DBException;
  
  public void createTable() throws DBException {
    internalCreate(this.fieldInfos);
  }
  
  public void open() throws DBException {
    if (this.active)
      return; 
    this.buffer = internalOpen();
    this.fieldInfos.refreshFinder();
    this.active = true;
  }
  
  public void append() throws DBException {
    this.buffer.clearBuffer();
  }
  
  public void post() throws DBException {
    postRecord(this.buffer);
  }
  
  public boolean isActive() {
    return this.active;
  }
  
  public void close() throws DBException {
    if (!this.active)
      return; 
    internalClose();
    this.active = false;
  }
  
  public int fieldIndexOf(String fieldName) {
    return this.fieldInfos.find(fieldName);
  }
  
  public void setBoolean(int index, boolean value) throws DBException {
    this.buffer.writeBoolean(this.fieldInfos.get(index - 1), value);
  }
  
  public void setBoolean(String fieldName, boolean value) throws DBException {
    this.buffer.writeBoolean(findField(fieldName), value);
  }
  
  public void setInt(int index, int value) throws DBException {
    this.buffer.writeInt(this.fieldInfos.get(index - 1), value);
  }
  
  public void setInt(String fieldName, int value) throws DBException {
    this.buffer.writeInt(findField(fieldName), value);
  }
  
  public void setLong(int index, long value) throws DBException {
    this.buffer.writeLong(this.fieldInfos.get(index - 1), value);
  }
  
  public void setLong(String fieldName, long value) throws DBException {
    this.buffer.writeLong(findField(fieldName), value);
  }
  
  public void setString(int index, String value) throws DBException {
    this.buffer.writeString(this.fieldInfos.get(index - 1), value);
  }
  
  public void setString(String fieldName, String value) throws DBException {
    this.buffer.writeString(findField(fieldName), value);
  }
  
  public void setDouble(int index, double value) throws DBException {
    this.buffer.writeDouble(this.fieldInfos.get(index - 1), value);
  }
  
  public void setDouble(String fieldName, double value) throws DBException {
    this.buffer.writeDouble(findField(fieldName), value);
  }
  
  public void setDate(int index, Date value) throws DBException {
    this.buffer.writeDate(this.fieldInfos.get(index - 1), value);
  }
  
  public void setDate(String fieldName, Date value) throws DBException {
    this.buffer.writeDate(findField(fieldName), value);
  }
  
  public void setNull(int index) throws DBException {
    this.buffer.writeNull(this.fieldInfos.get(index - 1));
  }
  
  public void setNull(String fieldName) throws DBException {
    this.buffer.writeNull(findField(fieldName));
  }
}
